import java.util.Scanner;
import java.io.File;
import java.io.IOException;

/**
 * This code reads a text file and is setup so that it can be run from the command line. 
 * It should not be in a package (just the "default" package).
 * 
 * To run it from the command line, in theory, you should be able to compile in Eclipse
 * and then go (using File Explorer) to the "bin" and run the class file: java ReadFile1 
 * (assuming the text file is in that location). Or, copy the class file and text file to
 * any other location and run it from there. However, it is possible (likely?) this will 
 * not run and you will see: "Error: A JNI error has occurred...". This indicates the JRE 
 * Eclipse is using is more recent than the one you are trying to run it under on your machine. 
 * There are ways to fix this (explained in Appendix in the text). However, another solution 
 * is to simply compile it from the command line: 
 * 
 *  1. Compile: javac ReadFile1.java
 *  2. Run: java ReadFile1, or: java ReadFile1 myTextFile.txt

 * To set the command-line argument (name of file) in Eclipse, choose: Run, Run configurations,
 * select your Java application on the left, then select the Arguments tab and then type in
 * the Program arguments.
 * 
 * @author dgibson
  */
public class ReadFile1 {
	public static void main(String[] args) throws IOException {
		boolean isBriefReport = true; // true=brief report, false=detailed
		String DEFAULT_FILE_NAME = "numbers.txt";
		
		String fileName = (args.length > 0 ? args[0] : DEFAULT_FILE_NAME);
		File inFile = new File(fileName);
		displayFileInfo(inFile, isBriefReport); 
		readFileAndDisplay(inFile);
	}
	
	private static void displayFileInfo(File inFile, boolean isBrief) throws IOException {
		System.out.println("File Info");
		System.out.println("---------------------------------------");
		if(isBrief) {
			System.out.println("             Name: " + inFile.getName());
			System.out.println("             Path: " + inFile.getPath());
			System.out.println("    Absolute file: " + inFile.getAbsoluteFile());
			System.out.println("    Does it exist? " + inFile.exists());
			System.out.println("   Can it be read? " + inFile.canRead());
			System.out.println("Can it be written? " + inFile.canWrite());
		} 
		else {
			System.out.println("             Name: " + inFile.getName());
			System.out.println("             Path: " + inFile.getPath());
			System.out.println("           Parent: " + inFile.getParent());
			System.out.println("      Parent file: " + inFile.getParentFile());
			System.out.println("   Default folder: " + getDefaultFolder(inFile));
			
			// The following 4 all give the same result on my computer (win 10)
			System.out.println("    Absolute file: " + inFile.getAbsoluteFile());
			System.out.println("    Absolute path: " + inFile.getAbsolutePath());
			System.out.println("   Canonical file: " + inFile.getCanonicalFile());
			System.out.println("   Canonical path: " + inFile.getCanonicalPath());
			// Derive the absolute path without the file name.
			System.out.println("Abs path(derived): " + getAbsolutePathWithoutFileName(inFile));
			
			String userDir = System.getProperty("user.dir");
			System.out.println("         user.dir: " + userDir);
			String userHome = System.getProperty("user.home");
			System.out.println("        user.home: " + userDir);

			System.out.println("    Does it exist? " + inFile.exists());
			System.out.println("   Can it be read? " + inFile.canRead());
			System.out.println("Can it be written? " + inFile.canWrite());
			System.out.println("Is it a directory? " + inFile.isDirectory());
			System.out.println("     Is it a file? " + inFile.isFile());
			System.out.println("     Is it hidden? " + inFile.isHidden());
			System.out.println("  Number of bytes: " + inFile.length() + " bytes");
			System.out.println("    Last modified: " + new java.util.Date(inFile.lastModified()));
		}
	}
	
	private static String getDefaultFolder(File inFile) {
		String absPath = inFile.getAbsolutePath();
		int posEndPath1 = absPath.lastIndexOf(inFile.getParent());
		int posEndPath2 = absPath.lastIndexOf(File.separator, posEndPath1-2);
//		System.out.println(posEndPath1 + ", " + posEndPath2 + ", " + absPath.length());
		String filePath = absPath.substring(posEndPath2+1,posEndPath1-1);
		return filePath;
	
	}
	
	private static String getAbsolutePathWithoutFileName(File inFile) {
		String absPath = inFile.getAbsolutePath();
		int posEndPath = absPath.lastIndexOf(File.separator);
		String filePath = absPath.substring(0,posEndPath+1);
		return filePath;
	}
	
	private static void readFileAndDisplay(File inFile) {
		System.out.println("\nFile Contents");
		System.out.println("---------------------------------------");
		if( inFile.exists()) {
			try {
				Scanner input = new Scanner( inFile );
				while( input.hasNext() ) {
					System.out.println(input.nextInt());
				}
				input.close();
			}
			catch( IOException e ) {
				System.out.println(e);
			}
		}
		else{
			System.out.println("File doesn't exist");
		}
		
	}

	
}

